#!/usr/bin/env python3
from ligo.gracedb.rest import GraceDb
import json
import logging

# setup verbose logs
logging.basicConfig(level=logging.INFO)


class GWTCGraceDB(GraceDb):
    """
    This candidate GraceDb client adds:
      1) a method for creating a new catalog
      2) a method for listing existing catalogs and versions
    """

    def gwtc_create(self, smap, number, comment=None):
        """Create a new GWTC entry.

        User must be a member of the catalog_managers group to create
        a gwtc entry.

        Args:
           smap (str | dict): path to a json-formatted file containing a catalog
               entry or a dictionary. An example of the format is below:
                   {
                     "S231002i": {
                         "pipelines": {
                             "MBTA": "G419572",
                             "spiir": "G419571",
                             "pycbc": "G419570",
                             "gstlal": "G419569"
                         },
                         "far": null,
                         "pastro": null
                     },
                     "S231002h": {
                         "pipelines": {
                             "MBTA": "G419568",
                             "spiir": "G419567",
                             "pycbc": "G419566",
                             "gstlal": "G419565"
                         },
                         "far": 1.8432234e-10,
                         "pastro": {
                             "BBH": 2.5953132292424023e-28,
                             "BNS": 0.9922100906247882,
                             "NSBH": 4.260287692039069e-29,
                             "Terrestrial": 0.007789909375211845
                         }
                     },
                     "S231002g": {
                         "pipelines": {
                             "MBTA": "G419564",
                             "spiir": "G419563",
                             "pycbc": "G419562",
                             "gstlal": "G419561"
                         },
                         "far": 3.23421e-23,
                         "pastro": null
                      }
                   }
               Note that superevent_id's and graceid's in the smap file must
               correspond to superevents and events on the live server.

               far values can be a number, or null

               pastro values must be a valid dict, or null

           number (str): identifying number of the catalog. Currently a slug.
               Example: number={'4', '4a', '4a'-1', etc}.

           comment (str, optional): an analyst comment to accompany the smap upload

        Returns:
            :class:`requests.models.Response`

        Raises:
            ligo.gracedb.exceptions.HTTPError: if the response has a status
                code >= 400.

        """

        # First open the smap file:

        if isinstance(smap, str):
            json_file = open(smap)
            smap_json = json.load(json_file)
            json_file.close()
        elif isinstance(smap, dict):
            smap_json = smap
        else:
            raise ValueError(
                "smap must be a string indicating a json filename or a dictionary"
            )

        # Verify that the number is a string. The server will verify that it's a
        # valid slug.
        if not isinstance(number, str):
            raise ValueError(f'"number" must be a str, not {type(number)}')

        # Construct the request uri:
        try:
            uri = str(self.links["gwtc"])
        except KeyError:
            raise NotImplementedError(
                f"gwtc functionality is not yet live on {self._service_url}"
            )

        json_request = {"number": number, "smap": smap_json, "comment": comment}

        return self.post(uri, json=json_request)

    def gwtc_get(self, number=None, version=None):
        """
        Retrieve a specific gwtc catalog based on number and version

        Args:
            number (str): identifying number of the catalog.

            version (int or str('latest')): positive integer version number of the
                catalog to retrieve. Entering 'latest' will pull the most recently
                uploaded version.

        Returns:
            :class:`requests.models.Response`

            A HTTPresponse with an individual catalog entry, if a specific version
            was specified.


        Note: version cannot be specified without specifying number.
        """

        # Try and construct the request URL to determine if gwtc is active
        # on the server.
        try:
            uri = str(self.templates["gwtc-version-detail"])
        except KeyError:
            raise NotImplementedError(
                f"gwtc functionality is not yet live on {self._service_url}"
            )

        # Verify 'number' and construct request URL.
        if number:
            if not isinstance(number, str):
                raise ValueError(f"number must be a str, not {type(number)}")
            elif version or version == 0:  # if the user has specified 'version'
                if (
                    (isinstance(version, int) and version <= 0)
                    or (isinstance(version, str) and version.lower() != "latest")
                    or (not isinstance(version, int) and not isinstance(version, str))
                ):
                    raise ValueError("version must be a positive integer or 'latest'")
                else:
                    uri = str(self.templates["gwtc-version-detail"]).format(
                        number=number, version=version
                    )
                    r = self.get(uri)
                    return r
            else:
                raise ValueError(
                    "both a number and a version must be supplied to " "gwtc_get"
                )

    def gwtc_list(self, number=None):
        """
        Retrieve an iterator of gwtc catalog dictionaries

        Args:
            number (str): identifying number of the catalog.

        Returns:
            :obj:`Iterator[dict]`

            An iterator which yields individual catalog dictionaries

        Note: not specifying a catalog number will retrieve an iterator with all
              available catalogs.

        """

        if number:
            if not isinstance(number, str):
                raise ValueError(f"number must be a str, not {type(number)}")
            else:
                uri = str(self.templates["gwtc-number-list"]).format(number=number)
        else:
            uri = str(self.links["gwtc"])

        # iterate over the results:
        while uri:
            response = self.get(uri).json()
            catalogs = response.get("results")
            uri = response.get("next")

            for catalog in catalogs:
                yield catalog

    def gwtc_numbers(self):
        """Return the unique catalog numbers

        Returns
        =======
        list
            The list of all unique catalog numbers (e.g. 4 for GWTC-4, etc)"""

        # Construct the request uri:
        try:
            uri = str(self.links["gwtc"])
        except KeyError:
            raise NotImplementedError(
                f"gwtc functionality is not yet live on {self._service_url}"
            )

        # Get the dictionary of catalog entries:
        catalog_dict = self.get(uri).json()

        # Construct a list of catalog numbers:
        catalog_numbers = [cat["number"] for cat in catalog_dict["results"]]

        # Return a unique list:
        return list(set(catalog_numbers))

    def gwtc_versions(self, number="4"):
        """Return the unique catalog versions

        Parameters
        ==========
        number : int, optional
            The GWTC number to act on, 4 for now

        Returns
        =======
        list
            The list of unique catalog versions for this catalog number
        """

        # Verify that the number is a string. The server will verify that it's a
        # valid slug.
        if not isinstance(number, str):
            raise ValueError(f'"number" must be a str, not {type(number)}')

        # Construct the request uri:
        try:
            uri = str(self.templates["gwtc-number-list"])
        except KeyError:
            raise NotImplementedError(
                f"gwtc functionality is not yet live on {self._service_url}"
            )

        # Get the dictionary of catalog number versions:
        versions_dict = self.get(uri.format(number=number)).json()

        # Construct and return a list of version numbers:
        return [cat["version"] for cat in versions_dict["results"]]



from gwtc.gwtc_gracedb import GWTCGraceDB
from .gwtc import update_far_and_pastro_in_smap, gwtc_update_pipeline_events
import logging
from optparse import OptionParser
import json
import time
import yaml

# setup verbose logs
logging.basicConfig(level=logging.INFO)


def get_super_events(client, g_events):
    out = {}
    retry = set()
    for gid in g_events:
        res = client.event(gid).json()
        if res["superevent"] is not None:
            out[res["superevent"]] = {
                "pipelines": {res["pipeline"]: gid},
                "far": None,
                "pastro": None,
            }
            logging.info(
                f"associated super event {res['superevent']} with gevent {gid}"
            )
        else:
            logging.info(f"No super event found for gevent {gid}")
            retry.add(gid)
    return out, retry


def parse_command_line():
    parser = OptionParser()
    parser.add_option(
        "--group",
        type="string",
        default="CBC",
        help="Set the group name to upload. default CBC",
    )
    parser.add_option(
        "--search",
        type="string",
        default="AllSky",
        help="Set the search name to upload. default AllSky",
    )
    parser.add_option("--pipeline", type="string", help="Set the pipeline. required")
    parser.add_option(
        "--proxy-path", type="string", help="Set the path to find the proxy file"
    )
    parser.add_option(
        "--service-url",
        type="string",
        default="https://gracedb-test.ligo.org/api/",
        help="Set the gracedb service url. Default: https://gracedb-test.ligo.org/api/",
    )
    parser.add_option(
        "--number", type="str", default="4", help='Set the catalog number: default "4"'
    )
    parser.add_option(
        "--reset",
        action="store_true",
        help="Assume the current gevents should replace the pipelines gwtc events by deleting gevents entries for superevents not in the provided set. This is useful for completely replacing events for a given pipeline but should not be used for just updating some gevents. NOTE: This doesn't delete any gevents from any database, it simply removes gevents mappings in the catalog.",
    )
    parser.add_option(
        "--in-yaml",
        type="str",
        help="Specify the yaml file containing input coinc files and pastros",
    )
    parser.add_option(
        "--out-yaml",
        type="str",
        help="Specify the yaml file to output the processed inputs",
    )
    options, filenames = parser.parse_args()
    assert len(filenames) == 0
    for option in ("pipeline",):
        if getattr(options, option.replace("-", "_")) is None:
            raise ValueError(f"--{option} is required")

    return options


def main():
    options = parse_command_line()
    if options.in_yaml is not None:
        with open(options.in_yaml) as f:
            cfg = yaml.safe_load(f.read())
    else:
        cfg = []

    client = GWTCGraceDB(service_url=options.service_url, cred=options.proxy_path)

    g_events = []
    for line in cfg:
        resp = client.createEvent(
            options.group,
            options.pipeline,
            line["coinc"],
            search=options.search,
            offline=True,
        )
        gid = resp.json()["graceid"]
        g_events.append(gid)
        line["gid"] = gid
        logging.info(f"Uploaded {g_events[-1]}")
        if line["pastro"]:
            with open(line["pastro"]) as f:
                client.writeLog(
                    gid,
                    "Submitted the p-astro file",
                    filename="p_astro.json",
                    filecontents=f.read(),
                    tagname="p_astro",
                )

    pmap = {}
    retry = set(g_events)
    while len(retry) > 0:
        time.sleep(1)
        out, retry = get_super_events(client, retry)
        pmap.update(out)

    pmap = update_far_and_pastro_in_smap(client, pmap)

    logging.info(f"Uploading\n{json.dumps(pmap, indent=4)}")
    resp = gwtc_update_pipeline_events(
        client, pmap, options.pipeline, options.number, reset=options.reset
    )
    logging.info(f"Created GWTC:\n{json.dumps(resp.json(), indent=4)}")

    if options.out_yaml is not None:
        with open(options.out_yaml, "w") as f:
            f.write(yaml.dump(cfg))



from gwtc.gwtc_gracedb import GWTCGraceDB
import logging
from optparse import OptionParser

# setup verbose logs
logging.basicConfig(level=logging.INFO)


def get_super_events(client, g_events):
    out = {}
    retry = set()
    for gid in g_events:
        res = client.event(gid).json()
        if res["superevent"] is not None:
            out[res["superevent"]] = {res["pipeline"]: gid}
            logging.info(
                f"associated super event {res['superevent']} with gevent {gid}"
            )
        else:
            logging.debug(f"No super event found for gevent {gid}")
            retry.add(gid)
    return out, retry


def parse_command_line():
    parser = OptionParser()
    parser.add_option(
        "--pipeline",
        type="string",
        help="Set the pipeline. If not set all pipeline g events will be returned",
    )
    parser.add_option(
        "--proxy-path", type="string", help="Set the path to find the proxy file"
    )
    parser.add_option(
        "--service-url",
        type="string",
        default="https://gracedb-test.ligo.org/api/",
        help="Set the gracedb service url. Default: https://gracedb-test.ligo.org/api/",
    )
    parser.add_option(
        "--number", type="str", default="4", help='Set the catalog number: default "4"'
    )
    parser.add_option(
        "--version",
        type="int",
        help="Set the version. If not given, latest will be used",
    )
    options, _ = parser.parse_args()

    return options


def main():
    options = parse_command_line()

    client = GWTCGraceDB(service_url=options.service_url, cred=options.proxy_path)

    smap = client.gwtc_get(options.number, version=options.version or "latest").json()[
        "gwtc_superevents"
    ]

    for sid in smap:
        for p, gid in smap[sid]["pipelines"].items():
            if options.pipeline is None or p == options.pipeline:
                event = client.event(gid).json()
                cfile = client.files(gid, filename="coinc.xml")
                fname = "%s-%s_%s-%d-0.xml" % (
                    "".join(sorted(event["instruments"].split(","))),
                    p.upper(),
                    gid,
                    int(event["gpstime"]),
                )
                logging.info(
                    "Retrieving g event %s for s event %s and pipeline %s\n\t...saving as %s"
                    % (gid, sid, p, fname)
                )
                with open(fname, "wb") as f:
                    f.write(cfile.read())



from . import _version

from .gwtc_update_pipeline_gevents import main as gwtc_update_pipeline_gevents
from .gwtc_diff import main as gwtc_diff
from .gwtc_get_gevent_coinc_files import main as gwtc_get_gevent_coinc_files
from .gwtc_create_from_query import main as gwtc_create_from_query

__version__ = _version.__version__



#!/usr/bin/env python3
from gwtc.gwtc_gracedb import GWTCGraceDB
from .gwtc import catalog_delta
import logging
from optparse import OptionParser


def parse_command_line():
    parser = OptionParser(usage="usage: %prog [options] version1 version2")
    parser.add_option(
        "--number",
        type="string",
        default="4",
        help='Set the catalog number: default "4"',
    )
    parser.add_option(
        "--proxy-path", type="string", help="Set the path to find the proxy file"
    )
    parser.add_option(
        "--service-url",
        type="string",
        default="https://gracedb-test.ligo.org/api/",
        help="Set the gracedb service url. Default: https://gracedb-test.ligo.org/api/",
    )
    options, versions = parser.parse_args()
    if len(versions) == 2:
        return options, int(versions[0]), int(versions[1])
    elif len(versions) == 0:
        return options, None, None
    else:
        raise ValueError(
            "Either provide two versions to compare or nothing to show the latest difference"
        )


def main():
    options, v1, v2 = parse_command_line()

    # setup verbose logs
    logging.basicConfig(level=logging.INFO)

    client = GWTCGraceDB(service_url=options.service_url, cred=options.proxy_path)

    logging.info(f"Calculating catalog diff from versions {v1} to {v2}")
    newsid, delsid, changedsid, diffstr = catalog_delta(
        client, v1, v2, number=options.number
    )
    logging.info(diffstr)
